@@ -5,20 +5,42 @@ import logging
 from oauthlib import oauth2
 from oauthlib.oauth2.rfc6749.endpoints import base
 from pyramid.response import Response
-from pyramid.compat import bytes_
+from pyramid.compat import native_
 
 log = logging.getLogger(__name__)
 
 OAUTH_PARAMS = (
     'access_token',
+    'client',
     'client_id',
     'client_secret',
     'code',
+    'extra_credentials',
     'grant_type',
+    'password',
+    'refresh_token',
+    'request_token',
     'response_type',
     'redirect_uri',
     'scope',
+    'scopes',
     'state',
+    'token',
+    'user',
+    'token_type_hint',
+    'username',
+
+    # OpenID Connect
+    'response_mode',
+    'nonce',
+    'display',
+    'prompt',
+    'claims',
+    'max_age',
+    'ui_locales',
+    'id_token_hint',
+    'login_hint',
+    'acr_values'
 )
 
 
@@ -155,22 +177,23 @@ def add_token_type(config, token_type, name='', **kwargs):
                   introspectables=(intr,), order=1)
 
 
-def add_oauth_param(config, name):  # pragma: no cover
+def add_oauth_param(config, name):
     def getter(request):
         return request.params.get(name)
     config.add_request_method(getter, str(name), reify=True)
 
 
-def duplicate_params(request):  # pragma: no cover
-    keys = request.params.keys()
+def duplicate_params(request):
+    keys = list(request.params)
     return [k for k in keys if keys.count(k) > 1]
 
-def oauth_response(result):  # pragma: no cover
+
+def oauth_response(result):
     headers, body, status = result
     return Response(body=body, status=status, headers={
-        bytes_(name, 'utf-8'): bytes_(value, 'utf-8')
+        native_(name, encoding='latin-1'): native_(value, encoding='latin-1')
         for name, value
-        in headers.iteritems()
+        in headers.items()
     })
 
 
@@ -191,9 +214,9 @@ def includeme(config):
     config.action('oauth server', register,
                   introspectables=(intr,), args=(config, server))
 
-    config.add_directive('add_grant_type', add_grant_type)
-    config.add_directive('add_response_type', add_response_type)
-    config.add_directive('add_token_type', add_token_type)
+    config.add_directive('add_grant_type', add_grant_type, True)
+    config.add_directive('add_response_type', add_response_type, True)
+    config.add_directive('add_token_type', add_token_type, True)
     config.add_directive('add_oauth_param', add_oauth_param)
 
     config.add_request_method(
@@ -229,7 +252,11 @@ def includeme(config):
         server.verify_request(request, scopes=scopes),
         str('verify_request'))
 
-    config.set_request_property(duplicate_params, str('duplicate_params'))
+    config.add_request_method(
+        lambda request: duplicate_params(request),
+        str("duplicate_params"),
+        property=True
+    )
 
     for name in OAUTH_PARAMS:
         config.add_oauth_param(str(name))
